unit LCDScreen;

////////////////////////////////////////////////////////////////////////////////
//
//  TLCDScreen / TLCDAnimator v2.0 (08/may/00)
//  written by Jacques VOIRIN
//  E-Mail: jacques.voirin@iname.com
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//   TLCDScreen is an alphanumeric LCD- like display with multidirectionnal
//   scrolling and smart animation features.
//
//   TLCDScreen can be fully customized throught its On/Off Pixels Intensity,
//   DotMatrix, PixelSize, etc. properties.
//   The caption is strored in Lines: TStrings.
//   In order to animate a TLCDScreen link it to a TLCDAnimator,
//   using its LCDAnimator property.
//   AnimationEnabled and AnimationDelay control the execution of the Animation
//   Code store in a TLCDAnimator.
//
//   For best look, use 65536 colors (16 bits) or higher.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//  TLCDScreen's props and methods list:
//  -----------------------------------
//
//   About: String               - Just for copyright;
//   AnimationDelay: Cardinal    - The smaller, the faster;
//   AnimationEnabled: Boolean   - Animation is set on or off;
//   AnimationRepeating: Boolean - If True, repeat the animation's code when done;
//   BorderColor: TColor     - The color of the border when BorderStyle is set to bsSingle;
//   BorderSpace: Byte       - The width in pixels beetween LCDdots and the border;
//   BorderStyle: TLCDBorder - bsRaised, bsLowered, bsSingle or bsNone;
//   CharSpacing: Byte       - The horz. width in pixels beetween LCD characters;
//   Color: TColor           - The main, background color of the component;
//   Cursor;
//   DotMatrix: TDotMatrix   - LCD's display look: mat5x7, mat5x8, mat7x9, mat9x12,
//                                                Hitachi, Hitachi2 or dos5x7;
//   Enabled;
//   Height;
//   Hint;
//   HorzScrollMode: TScrollMode - smIndependant or smDependant;
//                                 Only used with horizontal scroll directions.
//   Intensity: ShortInt     - The light power. [-127..127]
//                             'OnScreenOnPixColor := PixOnColor*Intensity'
//   LCDAnimator: TLCDAnimator - The animation code is store in it;
//   Left;
//   Lines: TStringList      - Where the text is strore. Can be edited at runtime
//                             using standard TStringList's functions;
//   LineSpacing: Byte       - The vert. width in pixels beetween LCD characters;
//   NoOfLines  : Byte       - The number of lines displayed;
//   NoOfChars  : Byte       - The number of characters displayed;
//   PixelHeight: Byte       - A read only prop in the general case.
//                             Only used when PixelSize is set to pixCustom;
//   PixelOff : TColor       - The color of switched on dots;
//   PixelOn  : TColor       - The color of switched off dots;
//   PixelShape: TPixelShape - The look of dots: psSquare, psShaped or psRound;
//   PixelSize: TPixelSize   - The size of one dot: pix1x1..pix16x16 or pixCustom;
//   PixelSpacing: Byte      - The vert.and horz.width in pixels beetween LCDdots;
//   PixelWidth: Byte        - A read only prop in the general case.
//                             Only used when PixelSize is set to pixCustom;
//   ShowHint;
//   Tag;
//   Top;
//   Visible;
//   Width;
//
//   OnClick;
//   OnDblClick;
//   OnDragDrop;
//   OnDragOver;
//   OnEndDrag;
//   OnMouseDown;
//   OnMouseMove;
//   OnMouseUp;
//
 //  procedure Reset(Value: TResetMode); TResetMode = (rmDisplay, rmCode, rmDisplayAndCode);
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//  TLCDAnimator's props and methods list:
//  --------------------------------------
//
//   About: String     - Just for copyright;
//   Code: TStrings    - The animation code; Can be edited throught the
//                       TLCDAnimatorEditor's Form;
//   CurrentLine: Byte - the number of the current code line ;
//
//   OnEndCode:      TEndCodeEvent;
//                   TEndCodeEvent = procedure (Sender: TLCDAnimator) of object;
//                   Event fire when code had proceed its last code line.
//
//   OnLineExecuted: TLineExecutedEvent;
//                   TLineExecutedEvent = procedure(Sender: TLCDAnimator; LineNumber: Word) of object;
//                   Event fire each time a line of the code had been proceeded.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//  History:
//
//   v1.0: (30/jan/00)
//         First public release
//
//   v1.1: (20/feb/00)
//         - New scrolling commands: ScrollDirection removed.
//             Use now ScrollHorz, ScrollVert: ShortInt [-127..127]
//         - Intensity implementation (Byte: [0..255])
//         - BorderSpace, CharSpacing, LineSpacing, NoOfLines, NoOfChars,
//           PixelSpacing and PixelWidth are now Byte type.
//         - CharSpacing and LineSpacing are now expressed width in pixels.
//         - Other cosmetic code design.
//
//   v1.2: (02/apr/00)
//         - New code for Intensity (ShortInt [-128..127])
//           Allow now 'inverse intensity'
//
//   v2.0: (01/may/00)
//         - Introducing of TLCDAnimator, the animation command for TLCDScreen
//           All 'Scroll' props and methods are now obsolet.
//           Use now AnimationDelay: Cardinal;
//                   AnimationEnabled: Boolean;
//                   AnimationRepeating: Boolean;
//                   and a TLCDAnimator to anime TLCDScreen;
//           Firt version of TLCDAnimatorCodeEditorForm.
//         - Remove all Dynamic Arrays: Less nice :(
//                                      but more compatible with old Delphi :))
//           Rewrite drawing/scrolling routines.
//           Removed internal FDisplayLines.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//  Know bugs:
//
//   none ( ! )
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//  The Strory and the Status of TLCDScreen
//  ---------------------------------------
//
//  TLCDScreen is based on an original code from Peter Czidlina's TLCDLabel v1.5a
//                               (petercz@swipnet.se)
//                               (http://hem2.passagen.se/ebcpecz/)
//
// The mother code for dots and characters drawing and Matrix.pas are Peter
// Czidlina's stuff.
// At the beginning, my idea was only to improve TLCDLabel.
// First, I try to implemente a true-multiline dimension. When done I add
// scrolling capacities to the component. I also recode drawing routines
// to improve speed.
// Days after days, improvments after improvments and e-mails after e-mails,
// it appears that I write a totally new component.
// Then, with the express agreement of Peter, I rename it and publish my work
// under my name.
//
// As Peter Czidlina's component was FREEWARE, it was easy for me to analyse his
// code and try to improve it.
//
// This is why TLCDScreen is also FREEWARE.
// However, use in commercial or shareware applications requires
// MY EXPRESS WRITTEN PERMISSION.
//
////////////////////////////////////////////////////////////////////////////////
//
// There is no direct compatibility between TLCDScreen and TLCDLabel because
// some properties had been renames, added or removed from the original code.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
// Installation note:
// ------------------
//    TLCDScreen use 'DsgnIntf.pas' for the About prop and the TLCDAnimator's
//    ComponentEditor and PropertyEditor.
//    'DsgnIntf.pas' can be find in '$(DELPHI)\Source\Toolsapi'
//    Then you must add it:
//                [Project/Options]
//                [Select Directories/Conditionnals]
//                Add [$(DELPHI)\Source\Toolsapi] in [SearchPath]
//
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//  Please notice bugs or problems, suggestions and improvements at
//
//  jacques.voirin@iname.com
//
////////////////////////////////////////////////////////////////////////////////


{$R LCDScreen.res}


interface


uses
  Windows, Classes, Graphics, Controls, Dialogs, ExtCtrls, DsgnIntf, Math,
  SysUtils, Forms, Matrix;


function NbOfThings(tempcode: string; Thing: Char): Integer;


const MaxSizeCodeInstructionArray = 255; //Removing dynamic Arrays...


type
  TLCDAnimator = class;

  TDotMatrix  = (mat5x7, mat5x8, mat7x9, mat9x12, Hitachi, Hitachi2, dos5x7 {, CustomFont});
  TPixelSize  = (pix1x1, pix2x2, pix3x3, pix4x4, pix5x5, pix6x6, pix7x7, pix8x8, pix9x9, pix10x10, pix11x11, pix12x12, pix13x13, pix14x14, pix15x15, pix16x16, pixCustom);
  TPixelShape = (psSquare, psShaped, psRound);
  TLCDBorder  = (bsRaised, bsLowered, bsSingle, bsNone);
  THorzScrollMode = (smIndependant, smDependant);
  TResetMode = (rmDisplay, rmCode, rmDisplayAndCode);

  TEndCodeEvent      = procedure(Sender: TLCDAnimator) of object;
  TLineExecutedEvent = procedure(Sender: TLCDAnimator; LineNumber: Word) of object;

  TCodeInstruction = record
                       Word: string[25];
                       Param: Integer;
                       end;

  CodeInstructionArrayBuilder = array[1..MaxSizeCodeInstructionArray] of TCodeInstruction; //Removing dynamic Arrays...


////////////////////////////////////////////////////////////////////////////////
//
//  TAbout Editor
//
////////////////////////////////////////////////////////////////////////////////

  TAboutProperty = class(TStringProperty)
  public
    function  GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;


////////////////////////////////////////////////////////////////////////////////
//
// TLCDAnimator
//
////////////////////////////////////////////////////////////////////////////////

  TLCDAnimator = class(TComponent)
  private
    FCode: TStrings;
    FCodeErrorFound: Boolean;
    FCurrentLine: Word;
    FOnEndCode: TEndCodeEvent;
    FOnLineExecuted: TLineExecutedEvent;
    function  GetAbout: string;
    procedure SetAbout(Value: string);

  protected
    procedure SetCode(Value: TStrings);

  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    property About: string read GetAbout write SetAbout stored False;
    property Code: TStrings read FCode write SetCode;
    property CodeErrorFound: Boolean read FCodeErrorFound write FCodeErrorFound default True;
    property CurrentLine: word read FCurrentLine write FCurrentLine;
    property OnEndCode: TEndCodeEvent read FOnEndCode write FOnEndCode;
    property OnLineExecuted: TLineExecutedEvent read FOnLineExecuted write FOnLineExecuted;
  end;


////////////////////////////////////////////////////////////////////////////////
//
// TLCDScreen
//
////////////////////////////////////////////////////////////////////////////////

  TLCDScreen = class(TGraphicControl)
  private
    FPixelSize: TPixelSize;             { Size of a LCD pixel (in screen pixels)    }
    FPixelShape: TPixelShape;           { Shape of a LCD pixel                      }
    FDotMatrix: TDotMatrix;             { Type of character matrix on the LCD       }
    FPixelSpacing: Byte;                { Space between each pixel in the matrix    }
    FCharSpacing: Byte;                 { Space between the characters on the LCD   }
    FLineSpacing: Byte;                 { Space between text lines on the display   }
    FBorderSpace: Byte;                 { Distance from the LCD border              }
    FNoOfLines: Byte;                   { Number of text lines on the LCD           }
    FNoOfChars: Byte;                   { Number of characters on a single line     }
    FColor: TColor;                     { LCD color                                 }
    FPixOnColor: TColor;                { LCD pixel ON color                        }
    FPixOffColor: TColor;               { LCD pixel OFF color                       }
    FIntensity: ShortInt;               { [-128..127] percentage of ligth           }
    FPixWidth: Byte;                    { Pixel widht                               }
    FPixHeight: Byte;                   { Pixel height                              }
    FBorderStyle: TLCDBorder;           { Border around the the component           }
    FBorderColor: TColor;               { Color of component border                 }
    FWidth: Integer;                    { Label width in pixels                     }
    FHeight: Integer;                   { Label height in pixels                    }
    charw, charh: Integer;              { Temp. storage of character sizes          }
    psx, psy: Byte;                     { Internal pixel width variables            }
    pix_x, pix_y: Byte;                 { Font size                                 }
    FLines: TStringList;                { The component's multilines text           }
    FtempLines: TStringList;            { Temp. and internal image of FLines        }
    FHorzScrollMode: THorzScrollMode;   { smIndependant or smDependant              }
    FTimer: TTimer;                     { Internal Timer                            }
    FVirginMatrixBitmap: TBitmap;       { Internal bitmap of a virgin matrix        }
    FBlackMatrixBitmap: TBitmap;        { Internal bitmap of a allOnPixels matrix   }
    FTrueOnColor : TColor;              { Internal color for OnPixels               }
    FTrueOffColor: TColor;              { Internal color for OffPixels              }
    FPanelColIndex: Byte;               { Internal index for PanelOn/Off effect     }
    FLCDAnimator: TLCDAnimator;         { The TLCDAnimator associated               }
    FAnimationEnabled: Boolean;         { Enable or disable the animation           }
    FAnimationDelay: Cardinal;          { in milliseconds; the smaller, the faster  }
    FAnimationRepeating: Boolean;       { What to do when all code lines had been processed }


    procedure SetPixelSize(Value: TPixelSize);
    procedure SetDotMatrix(Value: TDotMatrix);
    procedure SetPixelShape(Value: TPixelShape);
    procedure SetPixelSpacing(Value: Byte);
    procedure SetCharSpacing(Value: Byte);
    procedure SetLineSpacing(Value: Byte);
    procedure SetBorderSpace(Value: Byte);
    procedure SetNoOfLines(Value: Byte);
    procedure SetNoOfChars(Value: Byte);
    procedure SetColor(Value: TColor);
    procedure SetPixOnColor(Value: TColor);
    procedure SetPixOffColor(Value: TColor);
    procedure SetBorderStyle(Value: TLCDBorder);
    procedure SetBorderColor(Value: TColor);
    procedure SetPixelWidth(Value: Byte);
    procedure SetPixelHeight(Value: Byte);
    procedure DrawOneCharacter(BitMap: TBitMap; xpos, ypos: Integer; charindex: Byte);
    procedure DrawDisplayCharacters(BitMap: TBitMap);
    procedure UpdateInternalMatrixBitmap;
    procedure UpdateTrueColors;
    procedure SetCorrectSize;
    function  GetAbout: string;
    procedure SetAbout(Value: string);
    procedure SetHorzScrollMode(Value: THorzScrollMode);
    procedure SetIntensity(Value: ShortInt);
    procedure SetAnimationEnabled(Value: Boolean);
    procedure SetAnimationRepeating(Value: Boolean);
    procedure SetAnimationDelay(Value: Cardinal);
    procedure SetCorrectFTempLines;
    procedure SetLCDAnimator(Value: TLCDAnimator);

  protected
    procedure Paint; override;
    procedure SetLines(Value: TStringList);
    procedure LinesOnChange(Sender: TObject);
    procedure TimerOnTimer(Sender: TObject);
    procedure HorzScroll(Value: ShortInt);
    procedure VertScroll(Value: ShortInt);
    procedure InfoPanelOn(Sender: TObject);
    procedure InfoPanelOff(Sender: TObject);

  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    property  FontWidth: Byte read pix_x;
    property  FontHeight: Byte read pix_y;
    procedure Reset(Value: TResetMode);

  published
    property About: string read GetAbout write SetAbout stored False;
    property AnimationDelay: Cardinal read FAnimationDelay write SetAnimationDelay;
    property AnimationEnabled: Boolean read FAnimationEnabled write SetAnimationEnabled default False;
    property AnimationRepeating: Boolean read FAnimationRepeating write SetAnimationRepeating default False;
    property BorderColor: TColor read FBorderColor write SetBorderColor default clBlack;
    property BorderSpace: Byte read FBorderSpace write SetBorderSpace;
    property BorderStyle: TLCDBorder read FBorderStyle write SetBorderStyle default bsLowered;
    property CharSpacing: Byte read FCharSpacing write SetCharSpacing;
    property Color: TColor read FColor write SetColor default clSilver;
    property DotMatrix: TDotMatrix read FDotMatrix write SetDotMatrix default mat5x7;
    property Enabled;
    property HorzScrollMode: THorzScrollMode read FHorzScrollMode write SetHorzScrollMode;
    property Intensity: ShortInt read FIntensity write SetIntensity;
    property LCDAnimator: TLCDAnimator read FLCDAnimator write SetLCDAnimator;
    property Lines: TStringList read FLines write SetLines;
    property LineSpacing: Byte read FLineSpacing write SetLineSpacing;
    property NoOfLines: Byte read FNoOfLines write SetNoOfLines;
    property NoOfChars: Byte read FNoOfChars write SetNoOfChars;
    property PixelHeight: Byte read FPixHeight write SetPixelHeight;
    property PixelOff: TColor read FPixOffColor write SetPixOffColor default clGray;
    property PixelOn: TColor read FPixOnColor write SetPixOnColor default clBlack;
    property PixelShape: TPixelShape read FPixelShape write SetPixelShape default psSquare;
    property PixelSize: TPixelSize read FPixelSize write SetPixelSize;
    property PixelSpacing: Byte read FPixelSpacing write SetPixelSpacing;
    property PixelWidth: Byte read FPixWidth write SetPixelWidth;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;


const
  space_char = ' ';
  panel_char = char(255);


var
  P: ^CodeInstructionArrayBuilder; //Removing dynamic Arrays...


procedure Register;


implementation

////////////////////////////////////////////////////////////////////////////////
//
//  General Intrest Routines. Used for CodeEditor Synthax.
//
////////////////////////////////////////////////////////////////////////////////

function NbOfThings(tempcode: string; Thing: Char): Integer;
var i: Integer;
begin
  i := 0;
  while Pos(Thing, tempcode) <> 0
  do begin
       Delete(tempcode, 1, Pos(Thing, tempcode));
       Inc(i);
       end;
  NbOfThings := i;
end;


function ExtractWord(Value: String): String;
begin
  ExtractWord := Trim(Copy(Value, 1, Pos('(', Value) - 1));
end;


function ExtractParam(Value: String): Integer;
begin
  ExtractParam := StrtoInt(Copy(Value, Length(ExtractWord(Value)) + 2,
                           Pos(')', Value) - Length(ExtractWord(Value)) -2));
end;


procedure ExtractCode(tempcode: string; var CodeInstructions: array of TCodeInstruction;
                      NbOfElement: Byte);
var i,n: Integer;
    s,m: string;
begin
  tempcode := Trim(LowerCase(Copy(tempcode, 2, Length(tempcode) - 2)));
  for i := 0 to NbOfElement - 1
  do begin
       s := Trim(Copy(tempcode, 1, Pos(';', tempcode)));
       m := ExtractWord(s);
       n := ExtractParam(s);
       CodeInstructions[i].Word := m;
       CodeInstructions[i].Param := n;
       Delete(tempcode, 1, Length(s));
       tempcode := Trim(tempcode);
       end;
end;


////////////////////////////////////////////////////////////////////////////////
//
//  TAbout Editor routines.
//
////////////////////////////////////////////////////////////////////////////////

function TAboutProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;


procedure TAboutProperty.Edit;
begin
  MessageDlg('TLCDScreen / TLCDAnimator v2.0' + #13 + '(08/may/00)' + #13 + #13 +
    'written by Jacques Voirin (jacques.voirin@iname.com)' + #13 + #13 +
    'based on TLCDLabel v1.5a original source code' + #13 +
    'from Peter Czidlina (petercz@swipnet.se)', mtInformation, [mbOk], 0);
end;



////////////////////////////////////////////////////////////////////////////////
//
// Create and initialize component TLCDScreen.
//
////////////////////////////////////////////////////////////////////////////////

constructor TLCDScreen.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FLines := TStringList.Create;
  FLines.OnChange := LinesOnChange;

  FtempLines := TStringList.Create;
  FtempLines.Assign(FLines);

  FCharSpacing := 1;
  FLineSpacing := 1;
  FPixelSpacing := 1;
  FBorderSpace := 3;
  FNoOfLines := 1;
  FNoOfChars := 10;
  FBorderStyle := bsLowered;
  FBorderColor := clBlack;
  FColor := clSilver;
  FPixOnColor := clBlack;
  FPixOffColor := $00AAAAAA;
  FIntensity := 127;
  FPixelSize := pix2x2;

  UpdateTrueColors;

  FTimer := TTimer.Create(Self);
  FTimer.Interval := 250;
  FTimer.OnTimer := TimerOnTimer;
  FTimer.Enabled := False;

  FAnimationDelay := 250;
  FAnimationEnabled := False;
  FAnimationRepeating := False;

  FPanelColIndex := 0;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Remove component TLCDScreen.
//
////////////////////////////////////////////////////////////////////////////////

destructor TLCDScreen.Destroy;
begin
  FTimer.Destroy;
  FtempLines.Destroy;
  FLines.Destroy;
  FVirginMatrixBitmap.Destroy;
  FBlackMatrixBitmap.Destroy;
  inherited Destroy;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set Width and Height correct.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetCorrectSize;
var flag: boolean;
begin
  flag := False;

  if Width <> FWidth
  then begin
         flag := True;
         FWidth := Width;
         end;

  if Height <> FHeight
  then begin
         flag := True;
         FHeight := Height;
         end;

  if (Ord(pixCustom) = Ord(FPixelSize))
  then begin  { Custom size }
         psx := FPixWidth;
         psy := FPixHeight;
         end
  else begin  { Predefined width selected - make square pixels }
         psx := Ord(FPixelSize) + 1;
         psy := psx;
         FPixWidth := psx;
         FPixHeight := psy;
         end;

  charw := (pix_x * psx) + ((pix_x - 1) * FPixelSpacing);
  charh := (pix_y * psy) + ((pix_y - 1) * FPixelSpacing);

  if flag// or (csloading in ComponentState) //
         //or (csdesigning in ComponentState)
  then begin
         FNoOfChars := (Width  - 2 * FBorderSpace + FCharSpacing) div (charw + FCharSpacing);
         FNoOfLines := (Height - 2 * FBorderSpace + FLineSpacing) div (charh + FLineSpacing);
         if FNoOfChars < 1 then FNoOfChars := 1;
         if FNoOfLines < 1 then FNoOfLines := 1;
         end;

  Width := (FBorderSpace * 2) +                  { Distance to sides (there are two) }
           ((FCharSpacing) * (FNoOfChars - 1)) + { Spaces between charactes          }
           (charw * FNoOfChars) +                { The characters itself             }
           2;                                    { For the border                    }

  Height := (FBorderSpace * 2) +                 { Distance to top and bottom        }
           ((FLineSpacing) * (FNoOfLines - 1)) + { Spacing between lines             }
           (charh * FNoOfLines) +                { The lines                         }
           2;                                    { For the border                    }

  FWidth  := Width;
  FHeight := Height;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Create Virgin and Black Matrix.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.UpdateInternalMatrixBitmap;
var i, j: Byte;
    tx, ty: Integer;
begin
  if FVirginMatrixBitmap <> nil then FVirginMatrixBitmap.Destroy;
  if FBlackMatrixBitmap  <> nil then FBlackMatrixBitmap.Destroy;

  case FDotMatrix of       { Calculate the space taken by the character matrix }
     mat5x7,
     Hitachi: begin
                 pix_x := HITACHI_WIDTH;
                 pix_y := HITACHI_HEIGHT;
                 end;
     Hitachi2: begin
                 pix_x := HITACHI2_WIDTH;
                 pix_y := HITACHI2_HEIGHT;
                 end;
       mat5x8: begin
                 pix_x := MAT5X8_WIDTH;
                 pix_y := MAT5X8_HEIGHT;
                 end;
       mat7x9: begin
                 pix_x := MAT7X9_WIDTH;
                 pix_y := MAT7X9_HEIGHT;
                 end;
      mat9x12: begin
                 pix_x := MAT9X12_WIDTH;
                 pix_y := MAT9X12_HEIGHT;
                 end;
       dos5x7: begin
                 pix_x := DOS5X7_WIDTH;
                 pix_y := DOS5X7_HEIGHT;
                 end;
     end;

  SetCorrectSize;

  { Create FVirginMatrixBitmap: all pixels are set Off }

  FVirginMatrixBitmap := TBitmap.Create;
  FVirginMatrixBitmap.Height := charh;
  FVirginMatrixBitmap.Width  := charw;

  with FVirginMatrixBitmap.Canvas
  do begin
       Brush.Color := FColor;
       FillRect(Rect(0, 0, Width, Height));

       Pen.Color := FTrueOffColor;
       Brush.Color := FTrueOffColor;

       ty := 0;

       for i := 1 to pix_y
       do begin
            tx := 0;
            for j := 1 to pix_x
            do begin
                 if FPixelSize = pix1x1
                 then Pixels[tx, ty] := FTrueOffColor
                 else case FPixelShape of
            psSquare, psShaped:   { Standard square pixels and Pixels with shaped corners}
                               Rectangle(tx, ty, tx + psx, ty + psy);
                       psRound:   { Round pixels }
                               Ellipse(tx, ty, tx + psx, ty + psy);
                      end;
                 tx := tx + psx + FPixelSpacing;
                 end;
            ty := ty + psy + FPixelSpacing;
            end;
       end;

  { Create FBlackMatrixBitmap: all Pixels are set On }

  FBlackMatrixBitmap := TBitmap.Create;
  FBlackMatrixBitmap.Height := charh;
  FBlackMatrixBitmap.Width  := charw;

  BitBlt(FBlackMatrixBitmap.Canvas.Handle, 0, 0, FBlackMatrixBitmap.Width,
         FBlackMatrixBitmap.Height, FVirginMatrixBitmap.Canvas.Handle, 0, 0, srcCopy);

  with FBlackMatrixBitmap.Canvas
  do begin
       Pen.Color := FTrueOnColor;
       Brush.Color := FTrueOnColor;

       ty := 0;

       for i := 1 to pix_y-1
       do begin
            tx := 0;
            for j := 2 to pix_x
            do begin
                 if FPixelSize = pix1x1
                 then Pixels[tx, ty] := FTrueOnColor
                 else case FPixelShape of
            psSquare, psShaped:   { Standard square pixels and Pixels with shaped corners}
                               Rectangle(tx, ty, tx + psx, ty + psy);
                       psRound:   { Round pixels }
                               Ellipse(tx, ty, tx + psx, ty + psy);
                      end;
                 tx := tx + psx + FPixelSpacing;
                 end;
            ty := ty + psy + FPixelSpacing;
            end;
       end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Repaint the component.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.Paint;
var
  BitMap: TBitMap;
begin
  //if csDesigning in ComponentState then SetCorrectFTempLines;

  SetCorrectSize;

  if Visible
  then begin
  { Draw only real graphics if display is marked visible }
         BitMap := TBitMap.Create;
         BitMap.Height := Height;
         BitMap.Width := Width;

         with BitMap.Canvas
         do begin
  { Background fill }
              Brush.Color := FColor;
              FillRect(Rect(0, 0, Width, Height));
  { Border drawing }
              if FBorderStyle <> bsNone
              then begin
                     case FBorderStyle of
                        bsSingle: Pen.Color := FBorderColor;
                        bsRaised: Pen.Color := clBtnShadow;
                        bsLowered: Pen.Color := clBtnHighlight;
                        end;
                     PolyLine([Point(Width - 1, 0), Point(Width - 1, Height - 1), Point(-1, Height - 1)]);

                     case FBorderStyle of
                        bsLowered: Pen.Color := clBtnShadow;
                        bsRaised: Pen.Color := clBtnHighlight;
                        end;
                     PolyLine([Point(Width - 1, 0), Point(0, 0), Point(0, Height - 1)]);
                     end;

  { Characters drawing }
              if Enabled
              then begin
                     Brush.Color := FTrueOnColor; { Set Brush to Pixel-On color }
                     Pen.Color   := FTrueOnColor; { Set Pen to Pixel-On color   }
                     DrawDisplayCharacters(BitMap);
                     end;
  { Copy drawn characters to Window bitmap }
              BitBlt(Canvas.Handle, 0, 0, BitMap.Width, BitMap.Height, BitMap.Canvas.Handle, 0, 0, srcCopy);

              BitMap.Free;
              end;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Drawing routine for the display using FTempLines strings.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.DrawDisplayCharacters(BitMap: TBitMap);
var
  row, col: Byte;
  xpos, ypos: Integer;
begin
  ypos := FBorderSpace + 1;

  for row := 0 to FNoOfLines - 1
  do begin
       xpos := FBorderSpace + 1;

       for col := 1 to FNoOfChars
       do begin
            if (row < FtempLines.Count) and (Length(FtempLines[row]) >= col)
            then DrawOneCharacter(BitMap, xpos, ypos, Ord(Ftemplines[row][col]))
            else DrawOneCharacter(BitMap, xpos, ypos, Ord(space_char));
            xpos := xpos + charw + FCharSpacing;
            end;

       ypos := ypos + charh + FLineSpacing;
       end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Draw One Character ie Copy Vigin Matrix and then Draw only OnPixels.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.DrawOneCharacter(BitMap: TBitMap; xpos, ypos: Integer; charindex: Byte);
var
  i, j: Byte;
  tx, ty: Integer;
  PixelOn: Boolean;
begin
  if charindex = 255                      { char = 255: BlackChar; use FBlackMatrixBitmap }
  then BitBlt(BitMap.Canvas.Handle, xpos, ypos, FBlackMatrixBitmap.Width,
              FBlackMatrixBitmap.Height, FBlackMatrixBitmap.Canvas.Handle, 0, 0, srcCopy)
  else begin
         BitBlt(BitMap.Canvas.Handle, xpos, ypos, FVirginMatrixBitmap.Width,
                FVirginMatrixBitmap.Height, FVirginMatrixBitmap.Canvas.Handle, 0, 0, srcCopy); { Copy ViginMatrix to BitMap }
         if char(charindex) <> space_char { Useless to try to draw OnPixels with 'space' character! }
         then begin
                ty := ypos;

                for i := 1 to pix_y  { Find all OnPixels }
                do begin
                     tx := xpos;

                     for j := 1 to pix_x
                     do begin
                          PixelOn := False;

                          case FDotMatrix of
                   mat5x7: if Char5x7[charindex][i][j] = 1 then PixelOn := True;
                   mat5x8: if Char5x8[charindex][i][j] = 1 then PixelOn := True;
                  Hitachi: if CharHitachi[charindex][i][j] = 1 then PixelOn := True;
                 Hitachi2: begin                          // Use full height for character 194 - 223
                             if (charindex <= 193)
                             then begin
                                    if ((i <= 7) and (CharHitachi[charindex][i][j] = 1))
                                    then PixelOn := True; // Normal Hitachi
                                    end
                             else if CharHitachiExt[charindex][i][j] = 1
                                  then PixelOn := True;   // Extended height
                             end;
                   mat7x9: if Char7x9[charindex][i][j] = 1 then PixelOn := True;
                  mat9x12: if Char9x12[charindex][i][j] = 1 then PixelOn := True;
                   dos5x7: if CharDOS5x7[charindex][i][j] = 1 then PixelOn := True;
                          end;

                          if PixelOn { then draw the OnPixels }
                          then if FPixelSize = pix1x1
                               then BitMap.Canvas.Pixels[tx, ty] := FTrueOnColor
                               else case FPixelShape of
                                   psSquare:  { Standard square pixels}
                                            BitMap.Canvas.Rectangle(tx, ty, tx + psx, ty + psy);
                          psShaped, psRound:  { Pixels with shaped corners and Round pixels}
                                            BitMap.Canvas.Ellipse(tx, ty, tx + psx, ty + psy);
                                    end;
                          tx := tx + psx + FPixelSpacing;
                          end;
                     ty := ty + psy + FPixelSpacing;
                     end;
               end;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change type of dot matrix.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetDotMatrix(Value: TDotMatrix);
begin
  if Value <> FDotMatrix
  then begin
         FDotMatrix := Value;
         UpdateInternalMatrixBitmap;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change border style.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetBorderStyle(Value: TLCDBorder);
begin
  if Value <> FBorderStyle
  then begin
         FBorderStyle := Value;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change border color.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetBorderColor(Value: TColor);
begin
  if Value <> FBorderColor
  then begin
         FBorderColor := Value;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change shape of LCD pixels.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetPixelShape(Value: TPixelShape);
begin
  if Value <> FPixelShape
  then begin
         FPixelShape := Value;
         UpdateInternalMatrixBitmap;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change pixel spacing (distance between the pixels in the LCD).
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetPixelSpacing(Value: Byte);
begin
  if Value <> FPixelSpacing
  then begin
         FPixelSpacing := Value;
         UpdateInternalMatrixBitmap;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change character spacing (Distance between characters in the LCD).
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetCharSpacing(Value: Byte);
begin
  if Value <> FCharSpacing
  then begin
         FCharSpacing := Value;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change space between lines in a multi-line LCD.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetLineSpacing(Value: Byte);
begin
  if Value <> FLineSpacing
  then begin
         FLineSpacing := Value;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Change LCD pixel size.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetPixelSize(Value: TPixelSize);
begin
  if Value <> FPixelSize
  then begin
         FPixelSize := Value;
         UpdateInternalMatrixBitmap;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set space between the border and character array.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetBorderSpace(Value: Byte);
begin
  if Value <> FBorderSpace
  then begin
         FBorderSpace := Value;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set number of text lines on the LCD.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetNoOfLines(Value: Byte);
begin
  if Value < 1
  then MessageDlg('Display needs at least one line!', mtError, [mbOK], 0)
  else if Value <> FNoOfLines
       then begin
              FNoOfLines := Value;
              if FTimer.Enabled
              then LinesOnChange(Self)
              else Paint;
              end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set number of characters on one line (all lines are of same length).
//
//////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetNoOfChars(Value: Byte);
begin
  if Value < 1
  then MessageDlg('Display needs at least one character!', mtError, [mbOK], 0)
  else if Value <> FNoOfChars
       then begin
              FNoOfChars := Value;
              if FTimer.Enabled
              then LinesOnChange(Self)
              else Paint;
              end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set component color.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetColor(Value: TColor);
begin
  if Value <> FColor
  then begin
         FColor := Value;
         UpdateTrueColors;
         Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set pixel ON color.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetPixOnColor(Value: TColor);
begin
  if Value <> FPixOnColor
  then begin
         FPixOnColor := Value;
         UpdateTrueColors;
         if not FAnimationEnabled then Paint;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set pixel OFF color.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetPixOffColor(Value: TColor);
begin
  if Value <> FPixOffColor
  then begin
         FPixOffColor := Value;
         UpdateTrueColors;
         if not FAnimationEnabled then Paint;
         end;
end;


///////////////////////////////////////////////////////////////////////////////
//
// Set pixel ON color's intensity.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetIntensity(Value: ShortInt);
begin
  if Value <> FIntensity
  then begin
         FIntensity := Value;
         UpdateTrueColors;
         if not FAnimationEnabled then Paint;
         end;
end;


///////////////////////////////////////////////////////////////////////////////
//
// Calculate TrueOnColor.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.UpdateTrueColors;
var
  DeltaRed, DeltaGreen, DeltaBlue: Integer;
  R, G, B: Byte;
begin
  DeltaRed := GetRValue(ColorToRGB(FPixOffColor)) - GetRValue(ColorToRGB(FPixOnColor));
  DeltaGreen := GetGValue(ColorToRGB(FPixOffColor)) - GetGValue(ColorToRGB(FPixOnColor));
  DeltaBlue := GetBValue(ColorToRGB(FPixOffColor)) - GetBValue(ColorToRGB(FPixOnColor));
  R := MulDiv(129 - Abs(FIntensity), DeltaRed, 128);
  G := MulDiv(129 - Abs(FIntensity), DeltaGreen, 128);
  B := MulDiv(129 - Abs(FIntensity), DeltaBlue, 128);

  if FIntensity >= 0
  then begin
         FTrueOnColor := RGB(R, G, B);
         FTrueOffColor := FPixOffColor;
         end
  else begin
         FTrueOnColor := FPixOffColor;
         FTrueOffColor := RGB(R, G, B);
         end;

  UpdateInternalMatrixBitmap;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set pixel width.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetPixelWidth(Value: Byte);
begin
  if FPixelSize = pixCustom
  then if Value <> FPixWidth
       then if Value < 1
            then MessageDlg('Display pixel width must be 1 or greater!', mtError, [mbOk], 0)
            else begin
                   FPixWidth := Value;
                   UpdateInternalMatrixBitmap;
                   Paint;
                   end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set pixel height.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetPixelHeight(Value: Byte);
begin
  if FPixelSize = pixCustom
  then if Value <> FPixHeight
       then if Value < 1
            then MessageDlg('Display pixel height must be 1 or greate!', mtError, [mbOk], 0)
            else begin
                   FPixHeight := Value;
                   UpdateInternalMatrixBitmap;
                   Paint;
                   end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set FLines strings.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetLines(Value: TStringList);
begin
  FLines.Assign(Value);
end;


////////////////////////////////////////////////////////////////////////////////
//
// Update FtempLines with new FLines Strings.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.LinesOnChange(Sender: TObject);
begin
  FtempLines.Clear;
  FtempLines.Assign(FLines);
  SetCorrectFTempLines;
  Paint;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set FTempLines Correct.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetCorrectFTempLines;
var
  row, maxlength: Byte;
  tempstr: string;
begin
  if FtempLines.Count < FNoOfLines { Fill with '' Lines; use for Vert Scroll }
  then repeat
         FtempLines.Add('');
         until FtempLines.Count = FNoOfLines;

  if FhorzScrollMode = smIndependant
  then begin { Independant Lines }
         for row := 0 to FtempLines.Count - 1
         do if Length(FtempLines[row]) <> 0
            then begin
                   tempstr := FtempLines[row];

                   while Length(tempstr) <> FNoOfChars
                   do tempstr := tempstr + space_char;

                   FtempLines[row] := tempstr;
                   end;
         end

  else begin { Dedependant Lines }
         maxlength := FNoOfChars;

         for row := 0 to FtempLines.Count - 1
         do maxlength := Max(maxlength, Length(FtempLines[row]));

         for row := 0 to FtempLines.Count - 1
         do if Length(FtempLines[row]) <> 0
            then begin
                   tempstr := FtempLines[row];

                   while Length(tempstr) <> maxlength
                   do tempstr := tempstr + space_char;

                   FtempLines[row] := tempstr;
                   end;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set LCDAnimator.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetLCDAnimator(Value: TLCDAnimator);
var tmr: TModalResult;
begin
  if Value <> FLCDAnimator
  then begin
         tmr := mrIgnore;

         try
         if Value.CodeErrorFound and (csDesigning in ComponentState)
         then tmr := MessageDlg('Code synthax error(s) detected in this TLCDAnimator.' +
                                #13 +#10 + #13 + #10 + 'Continue anyway?',
                                mtWarning, [mbAbort, mbIgnore], 0);
         except;end;

         if tmr = mrIgnore
         then begin
                FPanelColIndex := 0;
                FTimer.OnTimer := TimerOnTimer;
                SetCorrectFTempLines;
                FLCDAnimator := Value;
                end;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set Scroll Mode.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetHorzScrollMode(Value: THorzScrollMode);
begin
  if Value <> FHorzScrollMode
  then begin
         FHorzScrollMode := Value;
         LinesOnChange(Self);
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set Animation Speed.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetAnimationDelay(Value: Cardinal);
begin
  if Value <> FAnimationDelay
  then begin
         FAnimationDelay := Value;
         FTimer.Interval := Value;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set Animation Active.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetAnimationEnabled(Value: Boolean);
begin
  if Value <> FAnimationEnabled
  then begin
         FAnimationEnabled := Value;
         if FAnimationEnabled
         then FTimer.Enabled := True
         else begin
                FTimer.Enabled := False;
                //LinesOnChange(Self);
                end;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set Cycling Animation.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.SetAnimationRepeating(Value: Boolean);
begin
  if Value <> FAnimationRepeating
  then FAnimationRepeating := Value;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Reset Method.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.Reset(Value: TResetMode);
begin
  case Value of
    rmDisplay: LinesOnChange(Self);
    rmCode   : FLCDAnimator.CurrentLine := 0;
    else begin {rmDisplayAndCode}
           LinesOnChange(Self);
           FLCDAnimator.CurrentLine := 0;
           end;
    end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// InfoPanelOn Method.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.InfoPanelOn(Sender: TObject);
var row: Byte;
    tempstr: string;
begin
  for row := 0 to FNoOfLines - 1
  do begin
       tempstr := FtempLines[row];

       tempstr[FPanelColIndex] := space_char;

       if row < FLines.Count
       then if FPanelColIndex > 1
            then if FPanelColIndex -1<= Length(FLines[row])
                 then tempstr[FPanelColIndex-1] := FLines[row][FPanelColIndex-1]
                 else tempstr[FPanelColIndex-1] := space_char;
       FtempLines[row] := tempstr;
       end;

  Paint;

  Inc(FPanelColIndex);

  if FPanelColIndex = FNoOfChars + 2
  then begin
         FTimer.OnTimer := TimerOnTimer;
         FPanelColIndex := 0;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// InfoPanelOff Method.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.InfoPanelOff(Sender: TObject);
var row: Byte;
    tempstr: string;
begin
  for row := 0 to FNoOfLines - 1
  do begin
       tempstr := FtempLines[row];

       if FPanelColIndex < Length(tempstr)
       then tempstr[FPanelColIndex] := space_char
       else tempstr := tempstr + space_char;

       if (FPanelColIndex > 1) and (FPanelColIndex - 1 < Length(tempstr))
       then tempstr[FPanelColIndex-1] := panel_char;

       FtempLines[row] := tempstr;
       end;

  Paint;

  Inc(FPanelColIndex);

  if FPanelColIndex = FNoOfChars + 2
  then begin
         FTimer.OnTimer := TimerOnTimer;
         FPanelColIndex := 0;
         end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// General Animation routine.
// Recalc FtempLines strings. Decode CurrentLine and execute it.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.TimerOnTimer(Sender: TObject);
var tempcode: string;
    i: Integer;
begin
  if FLCDAnimator <> nil
  then begin
         tempcode  :=  FLCDAnimator.Code[FLCDAnimator.CurrentLine];
         P := AllocMem(NbOfThings(tempcode, ';') * SizeOf(TCodeInstruction)); //Removing dynamic Arrays...
         ExtractCode(tempcode, P^, NbOfThings(tempcode, ';'));

         for i := 1 to NbOfThings(tempcode, ';')
         do begin
              if P^[i].Word = 'horzscroll' then HorzScroll(P^[i].Param);
              if P^[i].Word = 'vertscroll' then VertScroll(P^[i].Param);
              if P^[i].Word = 'setintensity' then SetIntensity(P^[i].Param);
              if P^[i].Word = 'animationdelay' then SetAnimationDelay(P^[i].Param);
              if P^[i].Word = 'gotoline' then FLCDAnimator.CurrentLine := Min(P^[i].Param, FLCDAnimator.Code.Count);
              if P^[i].Word = 'resetdisplay' then Reset(rmDisplay);
              if P^[i].Word = 'infopanelon'
              then begin
                     if FPanelColIndex = 0 then FPanelColIndex := 1;
                     FTimer.OnTimer := InfoPanelOn;
                     InfoPanelOn(Sender);
                     end;
              if P^[i].Word = 'infopaneloff'
              then begin
                     if FPanelColIndex = 0 then FPanelColIndex := 1;
                     FTimer.OnTimer := InfoPanelOff;
                     InfoPanelOff(Sender);
                     end;

              end;
         if Assigned(FLCDAnimator.FOnLineExecuted) then FLCDAnimator.FOnLineExecuted(FLCDAnimator, FLCDAnimator.CurrentLine);
         FLCDAnimator.CurrentLine := FLCDAnimator.CurrentLine + 1;
         if FLCDAnimator.CurrentLine = FLCDAnimator.Code.Count
         then begin
                FLCDAnimator.CurrentLine := 0;
                if not FAnimationRepeating then SetAnimationEnabled(False);
                if Assigned(FLCDAnimator.FOnEndCode) then FLCDAnimator.FOnEndCode(FLCDAnimator);
                end;
         FreeMem(P); //Removing dynamic Arrays...
         end;
  Paint;
end;


////////////////////////////////////////////////////////////////////////////////
//
// HorzScroll Routine.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.HorzScroll(Value: ShortInt);
var
  row: Byte;
  tempstr: string;
begin
  for row := 0 to FNoOfLines - 1
  do if row < FtempLines.Count
     then begin
            tempstr := FtempLines[row];
            if Value < 0
            then //sdRighttoLeft
                 tempstr := Copy(tempstr, Abs(Value)+1 , Length(tempstr) - Abs(Value) ) +
                            Copy(tempstr, 1, Abs(Value))
            else //sdLefttoRight
                 tempstr := Copy(tempstr, Length(tempstr) - Value+1, Value) +
                            Copy(tempstr, 1, Length(tempstr) - Value);
            FtempLines[row] := tempstr;
            end;
end;


////////////////////////////////////////////////////////////////////////////////
//
// VertScroll Routine.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDScreen.VertScroll(Value: ShortInt);
var
  i: Byte;
begin
  for i := 1 to Abs(Value)
  do if Value < 0
     then begin // sdUptoDown
            FtempLines.Add(FtempLines[0]);
            FtempLines.Delete(0);
            end
     else begin // sdDowntoUp
            FtempLines.Insert(0, FtempLines[FtempLines.Count - 1]);
            FtempLines.Delete(FtempLines.Count - 1);
            end;
end;



////////////////////////////////////////////////////////////////////////////////
//
// TLCDScreen About routines.
//
////////////////////////////////////////////////////////////////////////////////

function TLCDScreen.GetAbout: string;
begin
  GetAbout := 'About LCDScreen';
end;


procedure TLCDScreen.SetAbout(Value: string);
begin
  // just for syntax
end;



////////////////////////////////////////////////////////////////////////////////
//
// Create and initialize component TLCDAnimator.
//
////////////////////////////////////////////////////////////////////////////////

constructor TLCDAnimator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCode := TStringList.Create;
  FCodeErrorFound := True;
  FCurrentLine := 0;
end;


///////////////////////////////////////////////////////////////////////////////
//
// Remove component TLCDAnimator.
//
////////////////////////////////////////////////////////////////////////////////

destructor TLCDAnimator.Destroy;
begin
  FCode.Destroy;
  inherited Destroy;
end;


////////////////////////////////////////////////////////////////////////////////
//
// Set FCode strings.
//
////////////////////////////////////////////////////////////////////////////////

procedure TLCDAnimator.SetCode(Value: TStrings);
begin
  FCode.Assign(Value);
end;


////////////////////////////////////////////////////////////////////////////////
//
// TLCDAnimator About routines.
//
////////////////////////////////////////////////////////////////////////////////

function TLCDAnimator.GetAbout: string;
begin
  GetAbout := 'About LCDAnimator';
end;


procedure TLCDAnimator.SetAbout(Value: string);
begin
  // just for syntax
end;


////////////////////////////////////////////////////////////////////////////////
//
// TLCDScreen, TLCDAnimator and 'About' editor registration.
//
////////////////////////////////////////////////////////////////////////////////

procedure Register;
begin
  RegisterComponents('AddOn', [TLCDScreen, TLCDAnimator]);
  RegisterPropertyEditor(TypeInfo(string), TLCDScreen, 'About', TAboutProperty);
  RegisterPropertyEditor(TypeInfo(string), TLCDAnimator, 'About', TAboutProperty);
end;

end.

